package com.iqball.app.component

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawWithContent
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Rect
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.boundsInRoot
import androidx.compose.ui.layout.onGloballyPositioned
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import com.iqball.app.R
import com.iqball.app.domains.getPlayerInfo
import com.iqball.app.geo.toVector
import com.iqball.app.model.tactic.BallComponent
import com.iqball.app.model.tactic.ComponentId
import com.iqball.app.model.tactic.CourtType
import com.iqball.app.model.tactic.PlayerLike
import com.iqball.app.model.tactic.StepContent
import net.engawapg.lib.zoomable.ZoomState
import net.engawapg.lib.zoomable.rememberZoomState
import net.engawapg.lib.zoomable.zoomable

data class BasketCourtStates(
    val componentsOffsets: MutableMap<ComponentId, Offset>,
    val courtArea: MutableState<Rect>,
    val zoomState: ZoomState
)

@Composable
fun BasketCourt(
    content: StepContent,
    type: CourtType,
    modifier: Modifier,
    state: BasketCourtStates
) {
    val courtImg = when (type) {
        CourtType.Plain -> R.drawable.plain_court
        CourtType.Half -> R.drawable.half_court
    }

    val zoomState = state.zoomState
    val components = content.components

    val componentsOffset = state.componentsOffsets
    var courtArea by state.courtArea

    val playersPixelsRadius = LocalDensity.current.run { PlayerPieceDiameterDp.dp.toPx() / 2 }

    Box(
        modifier = modifier,
        contentAlignment = Alignment.Center
    ) {
        Box(
            modifier = Modifier
                .zoomable(zoomState),
            contentAlignment = Alignment.Center,
        ) {

            Image(
                painter = painterResource(id = courtImg),
                contentDescription = "court",
                modifier = Modifier
                    .background(Color.White)
            )

            Box(
                modifier = Modifier
                    .fillMaxSize()
                    .onGloballyPositioned {
                        if (courtArea == Rect.Zero)
                            courtArea = it.boundsInRoot()
                    }
                    .drawWithContent {
                        val relativeOffsets =
                            componentsOffset.mapValues { (it.value - courtArea.topLeft).toVector() }
                        drawActions(this, content, relativeOffsets, courtArea, playersPixelsRadius)
                        drawContent()
                    }
            ) {



                for (component in components) {
                    val componentModifier = Modifier
                        .onGloballyPositioned {
                            if (!componentsOffset.containsKey(component.id))
                                componentsOffset[component.id] = it.boundsInRoot().center
                        }
                    when (component) {
                        is PlayerLike -> {
                            val info = getPlayerInfo(component, content)
                            PlayerPiece(
                                player = info,
                                modifier = componentModifier
                                    .align(info.pos.toBiasAlignment())
                            )
                        }

                        is BallComponent -> BallPiece(
                            modifier = modifier
                                .align(
                                    component.pos
                                        .toPos()
                                        .toBiasAlignment()
                                )
                        )
                    }
                }
            }
        }
    }
}